﻿namespace LeadLoversMonitorApp.ViewModels
{
    using LLService;
    using System;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.ComponentModel;
    using System.Linq;
    using System.Windows.Input;
    using Utilities;
    using Xamarin.Forms;
    using Models;

    public class DashboardManager : INotifyPropertyChanged
    {
        public const double InitialPeriod = 7.1;
        public const int Maximum = 30;
        public const int MinimumPeriod = 2;
        private ObservableCollection<DashMaquina> fullQueryResult;
        private ObservableCollection<DashMaquina> filteredQuery;
        private string feedbackMessage;
        private bool isLoading;
        private double periodFilter;
        
        public DashboardManager(int machineId)
        {
            string token = UserData.Instance.Token;
            TimeSpan monthSpan = new TimeSpan(Maximum, 0, 0, 0);
            TimeSpan oneDay = new TimeSpan(1, 0, 0, 0);
            DateTime month = DateTime.Today.Subtract(monthSpan);
            this.filteredQuery = new ObservableCollection<DashMaquina>();
            this.fullQueryResult = new ObservableCollection<DashMaquina>();
            LLServiceClient lLClient = new LLServiceClient();
            lLClient.GetDashMaquinaCompleted += this.LLClient_GetDashMaquinaCompleted;
            lLClient.GetDashMaquinaAsync(token, machineId, month, DateTime.Today.Subtract(oneDay));
            this.IsLoading = true;
            this.FeedbackMessage = Resource.Strings.DashboardManager_FeedbackLoading;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        public ObservableCollection<DashMaquina> FilteredQuery
        {
            get
            {
                return this.filteredQuery;
            }

            private set
            {
                if (this.filteredQuery != value)
                {
                    this.filteredQuery = value;
                    this.NotifyPropertyChanged();
                }
            }
        }

        public ObservableCollection<DashMaquina> FullMachineCollection
        {
            get
            {
                return this.fullQueryResult;
            }
        }

        public int TotalCaptures
        {
            get
            {
                int totalCaptures = 0;
                foreach (DashMaquina machine in this.FilteredQuery)
                {
                    totalCaptures += machine.Captura;
                }

                return totalCaptures;
            }
        }

        public int TotalShoots
        {
            get
            {
                int totalShoots = 0;

                foreach (DashMaquina machine in this.FilteredQuery)
                {
                    totalShoots += machine.Disparos;
                }

                return totalShoots;
            }
        }

        public int TotalClicks
        {
            get
            {
                int totalClicks = 0;

                foreach (DashMaquina machine in this.FilteredQuery)
                {
                    totalClicks += machine.Cliques;
                }

                return totalClicks;
            }
        }

        public int TotalImported
        {
            get
            {
                int totalImported = 0;

                foreach (DashMaquina machine in this.FilteredQuery)
                {
                    totalImported += machine.Importacao;
                }

                return totalImported;
            }
        }

        public int TotalOpens
        {
            get
            {
                int totalOpens = 0;

                foreach (DashMaquina machine in this.FilteredQuery)
                {
                    totalOpens += machine.Aberturas;
                }

                return totalOpens;
            }
        }

        public int TotalError
        {
            get
            {
                int totalError = 0;

                foreach (DashMaquina machine in this.FilteredQuery)
                {
                    totalError += machine.Erros;
                }

                return totalError;
            }
        }

        public int TotalLeads
        {
            get
            {
                int totalLeads = 0;

                foreach (DashMaquina machine in this.FilteredQuery)
                {
                    totalLeads += machine.LeadsMaquina;
                }

                return totalLeads;
            }
        }

        public int TotalUnsub
        {
            get
            {
                int totalUnsub = 0;

                foreach (DashMaquina machine in this.FilteredQuery)
                {
                    totalUnsub += machine.Unsub;
                }

                return totalUnsub;
            }
        }

        public int TotalHard
        {
            get
            {
                int totalHard = 0;

                foreach (DashMaquina machine in this.FilteredQuery)
                {
                    totalHard += machine.Hard;
                }

                return totalHard;
            }
        }

        public int TotalSoft
        {
            get
            {
                int totalSoft = 0;

                foreach (DashMaquina machine in this.FilteredQuery)
                {
                    totalSoft += machine.Soft;
                }

                return totalSoft;
            }
        }

        public double TotalOpeningsRatio
        {
            get
            {
                double totalOpeningsRatio = ((double)this.TotalOpens / (double)this.TotalShoots) * 100;
                return totalOpeningsRatio;
            }
        }

        public double TotalClicksRatio
        {
            get
            {
                double totalClicksRatio = ((double)this.TotalClicks / (double)this.TotalOpens) * 100;
                return totalClicksRatio;
            }
        }

        public double TotalCapturesRatio
        {
            get
            {
                double totalCaptureRatio = ((double)this.TotalCaptures / (double)this.TotalClicks) * 100;
                return totalCaptureRatio;
            }
        }

        public double TotalUnsubRatio
        {
            get
            {
                double totalUnsubRatio = ((double)this.TotalUnsub / (double)this.TotalShoots) * 100;
                return totalUnsubRatio;
            }
        }

        public double TotalErrorRatio
        {
            get
            {
                double totalErrorRatio = ((double)this.TotalError / (double)this.TotalShoots) * 100;
                return totalErrorRatio;
            }
        }

        public bool IsLoading
        {
            get
            {
                return this.isLoading;
            }

            private set
            {
                this.isLoading = value;
                this.OnPropertyChanged("IsLoading");
            }
        }

        public string FeedbackMessage
        {
            get
            {
                return this.feedbackMessage;
            }

            set
            {
                this.feedbackMessage = value;
                this.OnPropertyChanged("FeedbackMessage");
            }
        }

        public double PeriodFilter
        {
            get
            {
                return this.periodFilter;
            }

            set
            {
                this.periodFilter = value;
                this.OnPropertyChanged("PeriodFilter");
                this.UpdateFilteredResult((int)value);
            }
        }

        public void UpdateFilteredResult(int value)
        {
            TimeSpan timeSpan = new TimeSpan(value + 1, 0, 0, 0);
            var limiteDate = DateTime.Today.Subtract(timeSpan);
            var filtered = this.fullQueryResult.Where(m => DateTime.Compare(m.Data, limiteDate) > 0);
            var orderedMachines = filtered.OrderBy(machine => machine.Data);
            this.FilteredQuery.Clear();
            foreach (DashMaquina machine in orderedMachines)
            {
                this.FilteredQuery.Add(machine);
            }

            this.NotifyPropertyChanged();
        }

        private void NotifyPropertyChanged()
        {
            this.OnPropertyChanged("FilteredQuery");
            this.OnPropertyChanged("TotalCaptures");
            this.OnPropertyChanged("TotalShoots");
            this.OnPropertyChanged("TotalClicks");
            this.OnPropertyChanged("TotalImported");
            this.OnPropertyChanged("TotalOpens");
            this.OnPropertyChanged("TotalError");
            this.OnPropertyChanged("TotalLeads");
            this.OnPropertyChanged("TotalUnsub");
            this.OnPropertyChanged("TotalHard");
            this.OnPropertyChanged("TotalSoft");
            this.OnPropertyChanged("TotalOpeningsRatio");
            this.OnPropertyChanged("TotalClicksRatio");
            this.OnPropertyChanged("TotalCapturesRatio");
            this.OnPropertyChanged("TotalUnsubRatio");
            this.OnPropertyChanged("TotalErrorRatio");
        }

        private void LLClient_GetDashMaquinaCompleted(object sender, LLService.GetDashMaquinaCompletedEventArgs e)
        {
            Device.BeginInvokeOnMainThread(() =>
            {
                if (e.Error != null || (e.Result.Count > 0 && !string.IsNullOrEmpty(e.Result[0].Mensagem)))
                {
                    this.FeedbackMessage = Resource.Strings.DashboardManager_FeedbackError;
                    return;
                }

                if (e.Result != null)
                {
                    this.IsLoading = false;
                    this.fullQueryResult = e.Result;
                    this.PeriodFilter = InitialPeriod;
                }
            });
        }

        private void OnPropertyChanged(string property)
        {
            if (this.PropertyChanged != null)
            {
                this.PropertyChanged(this, new PropertyChangedEventArgs(property));
            }
        }
    }
}
